#line 4 "grecs-lex.l"
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif	



#line 9 "grecs-lex.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

/* %not-for-header */

/* %if-c-only */
/* %if-not-reentrant */

/* %endif */
/* %endif */
/* %ok-for-header */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 6
#define YY_FLEX_SUBMINOR_VERSION 0
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* %if-c++-only */
/* %endif */

/* %if-c-only */
    
/* %endif */

/* %if-c-only */

/* %endif */

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
/* %if-c-only */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
/* %endif */

/* %if-tables-serialization */
/* %endif */
/* end standard C headers. */

/* %if-c-or-c++ */
/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

/* %endif */

/* %if-c++-only */
/* %endif */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define grecs_grecs_const const
#else
#define grecs_grecs_const
#endif

/* %not-for-header */

/* Returned upon end-of-file. */
#define YY_NULL 0
/* %ok-for-header */

/* %not-for-header */

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)
/* %ok-for-header */

/* %if-reentrant */
/* %endif */

/* %if-not-reentrant */

/* %endif */

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (grecs_grecs__start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((grecs_grecs__start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE grecs_grecs_restart(grecs_grecs_in  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(grecs_grecs__state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct grecs_grecs__buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t grecs_grecs__size_t;
#endif

/* %if-not-reentrant */
extern grecs_grecs__size_t grecs_grecs_leng;
/* %endif */

/* %if-c-only */
/* %if-not-reentrant */
extern FILE *grecs_grecs_in, *grecs_grecs_out;
/* %endif */
/* %endif */

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    #define YY_LINENO_REWIND_TO(ptr)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define grecs_grecs_less(n) \
	do \
		{ \
		/* Undo effects of setting up grecs_grecs_text. */ \
        int grecs_grecs_less_macro_arg = (n); \
        YY_LESS_LINENO(grecs_grecs_less_macro_arg);\
		*grecs_grecs__cp = (grecs_grecs__hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(grecs_grecs__c_buf_p) = grecs_grecs__cp = grecs_grecs__bp + grecs_grecs_less_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up grecs_grecs_text again */ \
		} \
	while ( 0 )

#define unput(c) grecs_grecs_unput( c, (grecs_grecs_text_ptr)  )

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct grecs_grecs__buffer_state
	{
/* %if-c-only */
	FILE *grecs_grecs__input_file;
/* %endif */

/* %if-c++-only */
/* %endif */

	char *grecs_grecs__ch_buf;		/* input buffer */
	char *grecs_grecs__buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	grecs_grecs__size_t grecs_grecs__buf_size;

	/* Number of characters read into grecs_grecs__ch_buf, not including EOB
	 * characters.
	 */
	grecs_grecs__size_t grecs_grecs__n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int grecs_grecs__is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int grecs_grecs__is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int grecs_grecs__at_bol;

    int grecs_grecs__bs_lineno; /**< The line count. */
    int grecs_grecs__bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int grecs_grecs__fill_buffer;

	int grecs_grecs__buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via grecs_grecs_restart()), so that the user can continue scanning by
	 * just pointing grecs_grecs_in at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */

/* %if-not-reentrant */

/* Stack of input buffers. */
static size_t grecs_grecs__buffer_stack_top = 0; /**< index of top of stack. */
static size_t grecs_grecs__buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * grecs_grecs__buffer_stack = 0; /**< Stack as an array. */
/* %endif */
/* %ok-for-header */

/* %endif */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (grecs_grecs__buffer_stack) \
                          ? (grecs_grecs__buffer_stack)[(grecs_grecs__buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (grecs_grecs__buffer_stack)[(grecs_grecs__buffer_stack_top)]

/* %if-c-only Standard (non-C++) definition */

/* %if-not-reentrant */
/* %not-for-header */

/* grecs_grecs__hold_char holds the character lost when grecs_grecs_text is formed. */
static char grecs_grecs__hold_char;
static grecs_grecs__size_t grecs_grecs__n_chars;		/* number of characters read into grecs_grecs__ch_buf */
grecs_grecs__size_t grecs_grecs_leng;

/* Points to current character in buffer. */
static char *grecs_grecs__c_buf_p = (char *) 0;
static int grecs_grecs__init = 0;		/* whether we need to initialize */
static int grecs_grecs__start = 0;	/* start state number */

/* Flag which is used to allow grecs_grecs_wrap()'s to do buffer switches
 * instead of setting up a fresh grecs_grecs_in.  A bit of a hack ...
 */
static int grecs_grecs__did_buffer_switch_on_eof;
/* %ok-for-header */

/* %endif */

void grecs_grecs_restart (FILE *input_file  );
void grecs_grecs__switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE grecs_grecs__create_buffer (FILE *file,int size  );
void grecs_grecs__delete_buffer (YY_BUFFER_STATE b  );
void grecs_grecs__flush_buffer (YY_BUFFER_STATE b  );
void grecs_grecs_push_buffer_state (YY_BUFFER_STATE new_buffer  );
void grecs_grecs_pop_buffer_state (void );

static void grecs_grecs_ensure_buffer_stack (void );
static void grecs_grecs__load_buffer_state (void );
static void grecs_grecs__init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER grecs_grecs__flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE grecs_grecs__scan_buffer (char *base,grecs_grecs__size_t size  );
YY_BUFFER_STATE grecs_grecs__scan_string (grecs_grecs_const char *grecs_grecs__str  );
YY_BUFFER_STATE grecs_grecs__scan_bytes (grecs_grecs_const char *bytes,grecs_grecs__size_t len  );

/* %endif */

void *grecs_grecs_alloc (grecs_grecs__size_t  );
void *grecs_grecs_realloc (void *,grecs_grecs__size_t  );
void grecs_grecs_free (void *  );

#define grecs_grecs__new_buffer grecs_grecs__create_buffer

#define grecs_grecs__set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        grecs_grecs_ensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            grecs_grecs__create_buffer(grecs_grecs_in,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->grecs_grecs__is_interactive = is_interactive; \
	}

#define grecs_grecs__set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        grecs_grecs_ensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            grecs_grecs__create_buffer(grecs_grecs_in,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->grecs_grecs__at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->grecs_grecs__at_bol)

/* %% [1.0] grecs_grecs_text/grecs_grecs_in/grecs_grecs_out/grecs_grecs__state_type/grecs_grecs_lineno etc. def's & init go here */
/* Begin user sect3 */

#define FLEX_DEBUG

typedef unsigned char YY_CHAR;

FILE *grecs_grecs_in = (FILE *) 0, *grecs_grecs_out = (FILE *) 0;

typedef int grecs_grecs__state_type;

extern int grecs_grecs_lineno;

int grecs_grecs_lineno = 1;

extern char *grecs_grecs_text;
#ifdef grecs_grecs_text_ptr
#undef grecs_grecs_text_ptr
#endif
#define grecs_grecs_text_ptr grecs_grecs_text

/* %% [1.5] DFA */

/* %if-c-only Standard (non-C++) definition */

static grecs_grecs__state_type grecs_grecs__get_previous_state (void );
static grecs_grecs__state_type grecs_grecs__try_NUL_trans (grecs_grecs__state_type current_state  );
static int grecs_grecs__get_next_buffer (void );
#if defined(__GNUC__) && __GNUC__ >= 3
__attribute__((__noreturn__))
#endif
static void grecs_grecs__fatal_error (grecs_grecs_const char msg[]  );

/* %endif */

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up grecs_grecs_text.
 */
#define YY_DO_BEFORE_ACTION \
	(grecs_grecs_text_ptr) = grecs_grecs__bp; \
/* %% [2.0] code to fiddle grecs_grecs_text and grecs_grecs_leng for grecs_grecs_more() goes here \ */\
	grecs_grecs_leng = (size_t) (grecs_grecs__cp - grecs_grecs__bp); \
	(grecs_grecs__hold_char) = *grecs_grecs__cp; \
	*grecs_grecs__cp = '\0'; \
/* %% [3.0] code to copy grecs_grecs_text_ptr to grecs_grecs_text[] goes here, if %array \ */\
	(grecs_grecs__c_buf_p) = grecs_grecs__cp;

/* %% [4.0] data tables for the DFA and the user's section 1 definitions go here */
#define YY_NUM_RULES 39
#define YY_END_OF_BUFFER 40
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct grecs_grecs__trans_info
	{
	flex_int32_t grecs_grecs__verify;
	flex_int32_t grecs_grecs__nxt;
	};
static grecs_grecs_const flex_int16_t grecs_grecs__accept[258] =
    {   0,
        0,    0,    2,    2,    0,    0,    0,    0,    0,    0,
       40,   37,   34,   35,   37,   15,   36,   19,   19,   37,
       18,   34,   15,    2,    4,    3,   39,   33,   39,   39,
       39,   25,   39,   38,   15,   39,   39,   15,   34,    0,
       20,    0,   15,   14,   19,    1,   17,    0,   18,   34,
        0,   15,   15,   15,   15,   15,   15,   15,    2,    3,
        3,    5,    0,   33,    0,    0,   25,    0,    0,    0,
       24,   17,    0,    0,   15,   15,   15,   22,   21,   19,
       17,   16,   17,    0,    0,    0,    0,   18,    0,    0,
        0,    0,    0,    0,    0,   15,   15,   15,   15,   15,

       15,   15,    0,   23,    0,    0,    0,   15,   15,   15,
        0,    0,    0,   28,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,   15,   15,   15,   15,   15,   15,
        0,    0,    0,    0,   15,   15,    0,    0,    0,   26,
        0,    0,    0,    0,    0,    0,    0,   15,   15,   15,
       15,   15,   15,   15,    0,    0,    0,   15,   15,   15,
        0,   31,    0,    0,    0,   27,    0,    0,    0,    0,
        0,    0,    0,   15,    6,   15,   15,   15,   15,   15,
       15,    0,    0,    0,    0,   15,    8,   15,   15,    0,
       29,    0,    0,    6,    0,    0,    0,    0,    0,    0,

       15,   15,   15,   15,    7,   15,    0,   32,    0,    8,
        0,    0,   15,    9,    0,   30,    0,    0,    0,    0,
        7,    0,   15,   13,   15,   11,   15,   15,    0,    9,
        0,   13,    0,   11,    0,    0,   15,   10,   15,   15,
        0,   10,    0,    0,   15,   15,   12,    0,    0,   12,
       15,    0,   15,    0,   15,    0,    0
    } ;

static grecs_grecs_const YY_CHAR grecs_grecs__ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    4,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    5,    1,    6,    7,    1,    1,    1,    1,    8,
        8,    9,    1,    8,   10,   11,   12,   13,   14,   14,
       14,   14,   14,   14,   14,   14,   14,   11,    8,   15,
        1,    1,    1,   11,   16,   16,   16,   16,   16,   16,
       16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
       16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
       11,   17,   11,    1,   18,    1,   19,   20,   21,   22,

       23,   16,   24,   16,   25,   16,   16,   26,   16,   27,
       28,   16,   16,   29,   16,   16,   30,   16,   31,   16,
       16,   16,    8,    1,    8,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static grecs_grecs_const YY_CHAR grecs_grecs__meta[32] =
    {   0,
        1,    2,    3,    1,    2,    4,    2,    1,    5,    6,
        7,    8,    9,    9,    1,   10,   11,   10,   10,   10,
       10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
       10
    } ;

static grecs_grecs_const flex_uint16_t grecs_grecs__base[306] =
    {   0,
        0,   30,   30,   31,  915,   35,   38,   40,   42,   56,
      917,  919,   46,  919,   30,  913,  919,    0,   44,  900,
       53,   65,   71,    0,  919,   66,  911,  919,   57,   60,
      919,  896,    0,  919,  909,   70,   79,   86,   99,   81,
      919,  908,  907,  919,    0,  919,  112,   89,   96,  119,
      128,    0,  158,   89,  107,  108,   68,  119,    0,  116,
      122,  919,  906,  919,  137,  126,  919,    0,  891,  904,
      919,  903,  139,  143,  150,  175,  162,  919,  919,    0,
      902,  919,  189,    0,  169,  205,  192,  172,    0,  209,
      884,  874,  875,  876,  881,  213,  222,  147,  153,   59,

      146,  197,  202,  919,    0,  226,  874,  239,  244,  210,
      892,  892,  257,  919,  893,  268,  883,  228,  263,  871,
      864,  871,  864,  861,  245,  229,  235,  217,  198,  247,
      249,  276,  281,  862,  281,  276,  298,  882,  884,  919,
      883,  879,  857,  855,  650,  631,  578,  282,  286,  136,
      264,  287,  304,  288,  163,  598,  579,  305,  309,  316,
        0,  919,  594,  583,  591,  919,  566,  317,  549,  541,
      538,  320,  505,  321,  919,  323,  328,  332,  294,  517,
      301,  516,  502,  333,  336,  337,  919,  339,  483,  474,
      919,  472,  341,  919,  343,  346,  348,  421,  392,  362,

      385,  384,  329,  383,  919,  351,  382,  919,  352,  919,
      353,  381,  380,  919,  379,  919,  378,  377,  356,  375,
      919,  349,  368,  919,  367,  919,  354,  358,  299,  919,
      295,  919,  263,  919,  359,  362,  258,  919,  362,  252,
      251,  919,  363,  250,  365,  200,  919,  366,  196,  919,
      164,  293,  371,  373,  134,   86,  919,  396,  407,  418,
      429,  440,  451,  457,  462,  472,  483,  494,  505,  516,
      527,  538,  545,  555,  563,  570,  580,   42,  590,  597,
      607,  618,  629,  640,  651,  662,  673,  684,  695,  706,
      717,  728,  739,  750,  761,  772,  783,  794,  805,  816,

      827,  838,  849,  860,  871
    } ;

static grecs_grecs_const flex_int16_t grecs_grecs__def[306] =
    {   0,
      257,    1,  258,  258,  259,  259,  260,  260,  261,  261,
      257,  257,  257,  257,  262,  263,  257,  264,  264,  257,
      265,  257,  263,  266,  257,  267,  268,  257,  268,  269,
      257,  270,  271,  257,  263,  257,  257,   23,  257,  262,
      257,  272,  263,  257,  273,  257,  274,  275,  276,  257,
      257,   23,  263,  263,  263,  263,  263,  263,  266,  267,
      267,  257,  268,  257,  268,  269,  257,  271,  270,  257,
      257,  277,  257,   51,   23,  263,  263,  257,  257,  273,
      277,  257,  274,  278,  275,  279,  275,  276,   51,  257,
      257,  257,  257,  257,  257,  263,  263,  263,  263,  263,

      263,  263,  268,  257,   74,  257,  257,  263,  263,  263,
      280,   85,  257,  257,  281,  279,  257,  257,  257,  257,
      257,  257,  257,  257,  282,  263,  263,  263,  263,  263,
      268,  257,  257,  257,  283,  263,  257,  280,  281,  257,
      284,  285,  257,  257,  257,  257,  257,  282,  286,  263,
      263,  263,  263,  263,  268,  287,  257,  283,  288,  263,
      137,  257,  289,  257,  284,  257,  285,  290,  257,  257,
      257,  257,  257,  286,  257,  286,  263,  263,  263,  291,
      263,  292,  287,  293,  257,  288,  257,  288,  294,  289,
      257,  295,  290,  257,  290,  257,  257,  257,  296,  257,

      297,  298,  263,  291,  257,  263,  292,  257,  293,  257,
      293,  299,  294,  257,  295,  257,  300,  301,  257,  296,
      257,  257,  297,  257,  298,  257,  302,  263,  299,  257,
      300,  257,  301,  257,  303,  257,  302,  257,  302,  304,
      303,  257,  303,  305,  302,  304,  257,  303,  305,  257,
      302,  303,  302,  303,  302,  303,    0,  257,  257,  257,
      257,  257,  257,  257,  257,  257,  257,  257,  257,  257,
      257,  257,  257,  257,  257,  257,  257,  257,  257,  257,
      257,  257,  257,  257,  257,  257,  257,  257,  257,  257,
      257,  257,  257,  257,  257,  257,  257,  257,  257,  257,

      257,  257,  257,  257,  257
    } ;

static grecs_grecs_const flex_uint16_t grecs_grecs__nxt[951] =
    {   0,
       12,   13,   14,   13,   13,   15,   16,   17,   18,   18,
       18,   19,   18,   18,   20,   21,   12,   21,   21,   21,
       21,   21,   21,   21,   21,   21,   21,   21,   21,   21,
       21,   22,   25,   25,   22,   41,   23,   28,   26,   26,
       31,   29,   31,   32,   34,   32,   42,   39,   35,   39,
       39,  111,   46,   36,   33,   47,   33,   37,   34,   64,
       37,   44,   38,   45,   45,   67,   50,   36,   39,   50,
       44,   51,   52,   44,   61,   52,   68,   62,   46,  128,
       73,   72,   65,   73,   53,   74,   41,   75,  242,   54,
       75,   44,  101,   55,   84,   56,   57,   42,   85,   76,

       39,   58,   39,   39,   80,   87,   80,   80,   98,   44,
       44,   77,   81,   81,   82,   81,   81,   81,   81,   81,
       50,   44,   39,   50,  257,   51,   81,  257,   81,   89,
       61,   67,   89,   62,  100,   99,  238,  102,   44,   64,
       73,   90,   68,   73,  105,   74,   91,  105,   44,   44,
       92,   75,   93,   94,   75,   44,  106,  177,   95,   96,
       44,  103,   96,   76,   44,   64,  238,  182,  107,  126,
       97,   97,  129,  112,   84,   77,  108,   44,  257,  108,
       80,  127,   80,   80,  253,   87,  110,  109,  109,   81,
       81,   82,   81,   81,   81,   81,   81,  257,  250,   44,

       44,  257,  247,   81,   64,   81,  113,  114,  257,  113,
      118,  115,   44,  118,   96,   44,  117,   96,  125,   44,
      153,  119,  119,   96,   44,  130,   96,  132,  131,  118,
      132,   44,  118,  142,   97,   97,  136,   44,  133,  133,
      108,   44,  152,  108,  135,  108,   44,   44,  108,   44,
      149,   64,  250,  242,  247,  150,  109,  109,  113,  114,
      238,  113,  151,  115,  118,  234,   44,  118,  117,  113,
      114,  155,  113,  154,  115,  119,  119,  132,   44,  117,
      132,  156,  132,   44,   44,  132,  159,  149,  175,   44,
       44,  176,  178,  133,  133,  242,   44,  232,  160,  161,

      162,  230,  161,   44,  163,  180,   44,   44,  180,  164,
      159,  187,  181,  254,  188,  203,  179,  189,   44,  194,
      189,  199,  195,  175,  199,  175,  176,  206,  176,  201,
       44,   44,  201,  202,   44,  210,  202,  212,  211,  187,
      212,  187,  188,  194,  188,  194,  195,  217,  195,  218,
      217,  227,  218,   44,  210,  210,  238,  211,  211,  240,
       44,  242,  240,  244,  238,  242,  244,  238,  242,  226,
      224,  239,  236,  238,  228,  242,  243,  221,  235,  234,
      232,  216,  214,  230,  208,  205,  226,  224,  222,  245,
      248,  251,  252,  255,  221,  256,   24,   24,   24,   24,

       24,   24,   24,   24,   24,   24,   24,   27,   27,   27,
       27,   27,   27,   27,   27,   27,   27,   27,   30,   30,
       30,   30,   30,   30,   30,   30,   30,   30,   30,   31,
       31,   31,   31,   31,   31,   31,   31,   31,   31,   31,
       40,   40,  219,   40,   40,   40,   40,   40,   40,   40,
       40,   43,   43,   43,   43,   43,   43,   43,   43,   43,
       43,   43,   45,   45,   45,   45,   45,   49,   49,   49,
       49,   49,   59,   59,  216,   59,  191,   59,   59,   59,
       59,   59,   59,   60,   60,  214,   60,   60,   60,   60,
       60,   60,   60,   60,   63,   63,   63,   63,   63,   63,

       63,   63,   63,   63,   63,   66,   66,  184,   66,   66,
       66,   66,   66,   66,   66,   66,   69,   69,  208,  205,
       69,   69,   69,   69,   69,   69,   69,   71,   71,  200,
       71,   71,   71,   71,   71,   71,   71,   71,   78,   78,
       78,   78,   78,   78,   78,   78,   78,   78,   78,   80,
       80,   80,   80,   80,   80,   83,   83,   83,   83,   83,
       83,   83,   83,   83,   83,   83,   86,  198,   86,  197,
      196,  168,   86,   86,   88,   88,   88,   88,   88,   88,
       81,   81,   81,   81,   81,   81,   81,   81,   81,   81,
       81,  116,  116,  166,  192,  116,  191,  116,  116,  116,

      138,  185,  138,  184,  173,  138,  138,  139,  139,  139,
      139,  139,  139,  139,  139,  139,  139,  139,  148,  148,
      148,  148,  148,  148,  148,  148,  148,  148,  148,  158,
      158,  158,  158,  158,  158,  158,  158,  158,  158,  158,
      165,  165,  165,  165,  165,  165,  165,  165,  165,  165,
      165,  167,  167,  172,  167,  167,  167,  167,  167,  167,
      167,  167,  174,  174,  174,  174,  174,  174,  174,  174,
      174,  174,  174,  183,  183,  171,  183,  183,  183,  183,
      183,  183,  183,  183,  186,  186,  186,  186,  186,  186,
      186,  186,  186,  186,  186,  190,  190,  190,  190,  190,

      190,  190,  190,  190,  190,  190,  193,  193,  193,  193,
      193,  193,  193,  193,  193,  193,  193,  204,  204,  204,
      204,  204,  204,  204,  204,  204,  204,  204,  207,  207,
      207,  207,  207,  207,  207,  207,  207,  207,  207,  209,
      209,  209,  209,  209,  209,  209,  209,  209,  209,  209,
      213,  213,  213,  213,  213,  213,  213,  213,  213,  213,
      213,  215,  215,  215,  215,  215,  215,  215,  215,  215,
      215,  215,  220,  220,  220,  220,  220,  220,  220,  220,
      220,  220,  220,  223,  223,  223,  223,  223,  223,  223,
      223,  223,  223,  223,  225,  225,  225,  225,  225,  225,

      225,  225,  225,  225,  225,  229,  229,  229,  229,  229,
      229,  229,  229,  229,  229,  229,  231,  231,  231,  231,
      231,  231,  231,  231,  231,  231,  231,  233,  233,  233,
      233,  233,  233,  233,  233,  233,  233,  233,  237,  237,
      237,  237,  237,  237,  237,  237,  237,  237,  237,  241,
      241,  241,  241,  241,  241,  241,  241,  241,  241,  241,
      246,  246,  246,  246,  246,  246,  246,  246,  246,  246,
      246,  249,  249,  249,  249,  249,  249,  249,  249,  249,
      249,  249,  170,  169,  168,  166,  140,  137,  157,  147,
      146,  145,  144,  143,  141,  140,  257,  137,  134,  124,

      123,  122,  121,  120,   82,   82,  104,   70,   64,   44,
       79,   44,   70,   64,   48,   44,  257,   28,   11,  257,
      257,  257,  257,  257,  257,  257,  257,  257,  257,  257,
      257,  257,  257,  257,  257,  257,  257,  257,  257,  257,
      257,  257,  257,  257,  257,  257,  257,  257,  257,  257
    } ;

static grecs_grecs_const flex_int16_t grecs_grecs__chk[951] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    3,    4,    2,   15,    2,    6,    3,    4,
        7,    6,    8,    7,    9,    8,   15,   13,    9,   13,
       13,  278,   19,    9,    7,   19,    8,   10,   10,   29,
       10,  100,   10,   21,   21,   30,   22,   10,   22,   22,
       57,   22,   23,   23,   26,   23,   30,   26,   36,  100,
       37,   36,   29,   37,   23,   37,   40,   38,  256,   23,
       38,   54,   57,   23,   48,   23,   23,   40,   48,   38,

       39,   23,   39,   39,   49,   48,   49,   49,   54,   55,
       56,   38,   47,   47,   47,   47,   47,   47,   47,   47,
       50,   58,   50,   50,   60,   50,   47,   60,   47,   51,
       61,   66,   51,   61,   56,   55,  255,   58,  150,   65,
       73,   51,   66,   73,   74,   73,   51,   74,  101,   98,
       51,   75,   51,   51,   75,   99,   74,  150,   51,   53,
       53,   65,   53,   75,   77,  155,  251,  155,   74,   98,
       53,   53,  101,   85,   85,   75,   76,   76,   85,   76,
       88,   99,   88,   88,  251,   85,   77,   76,   76,   83,
       83,   83,   83,   83,   83,   83,   83,   87,  249,  102,

      129,   87,  246,   83,  103,   83,   86,   86,   87,   86,
       90,   86,  110,   90,   96,   96,   86,   96,   96,  128,
      129,   90,   90,   97,   97,  102,   97,  106,  103,  118,
      106,  126,  118,  118,   97,   97,  110,  127,  106,  106,
      108,  108,  128,  108,  108,  109,  109,  125,  109,  130,
      125,  131,  244,  241,  240,  126,  109,  109,  113,  113,
      237,  113,  127,  113,  119,  233,  151,  119,  113,  116,
      116,  131,  116,  130,  116,  119,  119,  132,  136,  116,
      132,  132,  133,  135,  148,  133,  135,  148,  149,  152,
      154,  149,  151,  133,  133,  252,  179,  231,  136,  137,

      137,  229,  137,  181,  137,  153,  153,  158,  153,  137,
      158,  159,  154,  252,  159,  179,  152,  160,  160,  168,
      160,  172,  168,  174,  172,  176,  174,  181,  176,  177,
      177,  203,  177,  178,  178,  184,  178,  185,  184,  186,
      185,  188,  186,  193,  188,  195,  193,  196,  195,  197,
      196,  203,  197,  206,  209,  211,  227,  209,  211,  228,
      228,  235,  228,  236,  239,  243,  236,  245,  248,  225,
      223,  227,  222,  253,  206,  254,  235,  220,  219,  218,
      217,  215,  213,  212,  207,  204,  202,  201,  200,  239,
      243,  245,  248,  253,  199,  254,  258,  258,  258,  258,

      258,  258,  258,  258,  258,  258,  258,  259,  259,  259,
      259,  259,  259,  259,  259,  259,  259,  259,  260,  260,
      260,  260,  260,  260,  260,  260,  260,  260,  260,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      262,  262,  198,  262,  262,  262,  262,  262,  262,  262,
      262,  263,  263,  263,  263,  263,  263,  263,  263,  263,
      263,  263,  264,  264,  264,  264,  264,  265,  265,  265,
      265,  265,  266,  266,  192,  266,  190,  266,  266,  266,
      266,  266,  266,  267,  267,  189,  267,  267,  267,  267,
      267,  267,  267,  267,  268,  268,  268,  268,  268,  268,

      268,  268,  268,  268,  268,  269,  269,  183,  269,  269,
      269,  269,  269,  269,  269,  269,  270,  270,  182,  180,
      270,  270,  270,  270,  270,  270,  270,  271,  271,  173,
      271,  271,  271,  271,  271,  271,  271,  271,  272,  272,
      272,  272,  272,  272,  272,  272,  272,  272,  272,  273,
      273,  273,  273,  273,  273,  274,  274,  274,  274,  274,
      274,  274,  274,  274,  274,  274,  275,  171,  275,  170,
      169,  167,  275,  275,  276,  276,  276,  276,  276,  276,
      277,  277,  277,  277,  277,  277,  277,  277,  277,  277,
      277,  279,  279,  165,  164,  279,  163,  279,  279,  279,

      280,  157,  280,  156,  147,  280,  280,  281,  281,  281,
      281,  281,  281,  281,  281,  281,  281,  281,  282,  282,
      282,  282,  282,  282,  282,  282,  282,  282,  282,  283,
      283,  283,  283,  283,  283,  283,  283,  283,  283,  283,
      284,  284,  284,  284,  284,  284,  284,  284,  284,  284,
      284,  285,  285,  146,  285,  285,  285,  285,  285,  285,
      285,  285,  286,  286,  286,  286,  286,  286,  286,  286,
      286,  286,  286,  287,  287,  145,  287,  287,  287,  287,
      287,  287,  287,  287,  288,  288,  288,  288,  288,  288,
      288,  288,  288,  288,  288,  289,  289,  289,  289,  289,

      289,  289,  289,  289,  289,  289,  290,  290,  290,  290,
      290,  290,  290,  290,  290,  290,  290,  291,  291,  291,
      291,  291,  291,  291,  291,  291,  291,  291,  292,  292,
      292,  292,  292,  292,  292,  292,  292,  292,  292,  293,
      293,  293,  293,  293,  293,  293,  293,  293,  293,  293,
      294,  294,  294,  294,  294,  294,  294,  294,  294,  294,
      294,  295,  295,  295,  295,  295,  295,  295,  295,  295,
      295,  295,  296,  296,  296,  296,  296,  296,  296,  296,
      296,  296,  296,  297,  297,  297,  297,  297,  297,  297,
      297,  297,  297,  297,  298,  298,  298,  298,  298,  298,

      298,  298,  298,  298,  298,  299,  299,  299,  299,  299,
      299,  299,  299,  299,  299,  299,  300,  300,  300,  300,
      300,  300,  300,  300,  300,  300,  300,  301,  301,  301,
      301,  301,  301,  301,  301,  301,  301,  301,  302,  302,
      302,  302,  302,  302,  302,  302,  302,  302,  302,  303,
      303,  303,  303,  303,  303,  303,  303,  303,  303,  303,
      304,  304,  304,  304,  304,  304,  304,  304,  304,  304,
      304,  305,  305,  305,  305,  305,  305,  305,  305,  305,
      305,  305,  144,  143,  142,  141,  139,  138,  134,  124,
      123,  122,  121,  120,  117,  115,  112,  111,  107,   95,

       94,   93,   92,   91,   81,   72,   70,   69,   63,   43,
       42,   35,   32,   27,   20,   16,   11,    5,  257,  257,
      257,  257,  257,  257,  257,  257,  257,  257,  257,  257,
      257,  257,  257,  257,  257,  257,  257,  257,  257,  257,
      257,  257,  257,  257,  257,  257,  257,  257,  257,  257
    } ;

static grecs_grecs__state_type grecs_grecs__last_accepting_state;
static char *grecs_grecs__last_accepting_cpos;

extern int grecs_grecs__flex_debug;
int grecs_grecs__flex_debug = 1;

static grecs_grecs_const flex_int16_t grecs_grecs__rule_linenum[39] =
    {   0,
      171,  173,  174,  175,  177,  181,  186,  194,  200,  210,
      212,  215,  218,  225,  227,  228,  230,  233,  235,  241,
      246,  252,  258,  260,  261,  269,  270,  271,  272,  273,
      274,  279,  282,  306,  308,  309,  310,  319
    } ;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define grecs_grecs_more() grecs_grecs_more_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *grecs_grecs_text;
#line 1 "grecs-lex.l"
/* grecs - Gray's Extensible Configuration System       -*- c -*- */

#line 9 "grecs-lex.l"
/* grecs - Gray's Extensible Configuration System
   Copyright (C) 2007-2021 Sergey Poznyakoff

   Grecs is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3 of the License, or (at your
   option) any later version.

   Grecs is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with Grecs. If not, see <http://www.gnu.org/licenses/>. */

#include <grecs.h>
#include <grecs-gram.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/wait.h>	
#include <glob.h>
#include <signal.h>
#include <wordsplit.h>

/*
 * Locus tracking
 */
struct grecs_locus_point grecs_current_locus_point;   /* Input file location */

#define YY_USER_ACTION do {						\
		if (YYSTATE == INITIAL || YYSTATE == PP) {		\
			grecs_grecs_lloc.beg = grecs_current_locus_point;		\
			grecs_grecs_lloc.beg.col++;				\
		}							\
  		grecs_current_locus_point.col += grecs_grecs_leng;		\
 		grecs_grecs_lloc.end = grecs_current_locus_point;			\
   	} while (0);

/*
 * Redefine default error reporting function.
 */
#define YY_FATAL_ERROR(t) grecs_error(NULL, errno, "%s", t)

/*
 * Preprocessing and synclines.
 */
static int initial_state; /* Lexer initial state.
                           * Set to INITIAL when parsing the file, and to PP
                           * when generating preprocessed output.
			   */
static int emit_syncline; /* If not null, emit syncline before next output
			   * line.
			   */
static int beginning_of_line; /* True if output is at the beginning of line */

/*
 * Echo LEN bytes from TEXT to grecs_grecs_out.
 */
static void
echo(char const *text, size_t len)
{
	if (len == 0) return;
	/* Check if we should print a syncline */
	if (beginning_of_line && emit_syncline) {
		fprintf(grecs_grecs_out, "#line %lu \"%s\"\n",
			(unsigned long)grecs_grecs_lloc.end.line,
			grecs_grecs_lloc.end.file);
		emit_syncline = 0;
	}
	/* Actually echo the text. */
	fwrite(text, len, 1, grecs_grecs_out);
	/* Determine the beginning of line status. */
	beginning_of_line = text[len-1] == '\n';
}
/* Inform flex that it should use echo. */
#define ECHO echo(grecs_grecs_text, grecs_grecs_leng)

/*
 * String concatenation support.
 *
 * String concatenation is a legacy feature of Grecs parser that instructs
 * it to concatenate adjacent quoted strings.  It is enabled if the
 * GRECS_OPTION_QUOTED_STRING_CONCAT bit is set in parser options.  This
 * bit causes the lexer to qualify each double-quoted string as QSTRING.
 */
#define qstring()							\
	((grecs_parser_options & GRECS_OPTION_QUOTED_STRING_CONCAT)	\
	 ? QSTRING : STRING)

/*
 * Lexical tie-in variables.
 */
extern int grecs_grecs_grecs_error_suppress;

/*
 * Here-document support.
 */
static char *multiline_delimiter;       /* Here-document delimiter text. */
static size_t multiline_delimiter_len;  /* Its length. */
static int multiline_unescape;          /* Unescape here-document contents */
static int (*char_to_strip)(char);      /* Strips matching characters of each
					   here-document line */

static void multiline_begin(char *);
static void multiline_add(char *);
static char *multiline_strip_tabs(char *text);
static int isemptystr(int off);

/*
 * Forward declarations.
 */
static int ident(void);
static void qstring_locus_fixup(void);
static int parse_include(char *text);
static int push_source(const char *name, int once);
static int pop_source(void);
static void ctx_abort(void);

/*
 * The report_diag should be called only when grecs_grecs_text begins with the following
 * prefix:
 *       ^[ \t]*#[ \t]*.{LEN}
 * It returns a pointer to grecs_grecs_text past the prefix.
 */
static char *
report_diag(size_t len)
{
	char *p = strchr(grecs_grecs_text, '#');
	p += strspn(p, " \t") + len;
	p += strspn(p, " \t");
	grecs_grecs_text[grecs_grecs_leng-1] = 0;
	if (p[0] == '"' && grecs_grecs_text[grecs_grecs_leng-2] == '"') {
		char *q, *r;
		
		grecs_grecs_text[grecs_grecs_leng-2] = 0;
		p++;
		q = r = p;
		do {
			if (*r == '\\') {
				int c = wordsplit_c_unquote_char(*++r);
				*q = c ? c : *r;
			} else
				*q = *r++;
		} while (*q++ == 0);
	}
	return p;
}

#line 1022 "grecs-lex.c"

#define INITIAL 0
#define COMMENT 1
#define ML 2
#define STR 3
#define PP 4

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
/* %if-c-only */
#include <unistd.h>
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

/* %if-c-only Reentrant structure and macros (non-C++). */
/* %if-reentrant */
/* %if-c-only */

static int grecs_grecs__init_globals (void );

/* %endif */
/* %if-reentrant */
/* %endif */
/* %endif End reentrant structures and macros. */

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int grecs_grecs_lex_destroy (void );

int grecs_grecs_get_debug (void );

void grecs_grecs_set_debug (int debug_flag  );

YY_EXTRA_TYPE grecs_grecs_get_extra (void );

void grecs_grecs_set_extra (YY_EXTRA_TYPE user_defined  );

FILE *grecs_grecs_get_in (void );

void grecs_grecs_set_in  (FILE * _in_str  );

FILE *grecs_grecs_get_out (void );

void grecs_grecs_set_out  (FILE * _out_str  );

grecs_grecs__size_t grecs_grecs_get_leng (void );

char *grecs_grecs_get_text (void );

int grecs_grecs_get_lineno (void );

void grecs_grecs_set_lineno (int _line_number  );

/* %if-bison-bridge */
/* %endif */

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int grecs_grecs_wrap (void );
#else
extern int grecs_grecs_wrap (void );
#endif
#endif

/* %not-for-header */

#ifndef YY_NO_UNPUT
    
#endif
/* %ok-for-header */

/* %endif */

#ifndef grecs_grecs_text_ptr
static void grecs_grecs__flex_strncpy (char *,grecs_grecs_const char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int grecs_grecs__flex_strlen (grecs_grecs_const char * );
#endif

#ifndef YY_NO_INPUT
/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */

#ifdef __cplusplus
static int grecs_grecs_input (void );
#else
static int input (void );
#endif
/* %ok-for-header */

/* %endif */
#endif

/* %if-c-only */

/* %endif */

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* %if-c-only Standard (non-C++) definition */
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( grecs_grecs_text, grecs_grecs_leng, 1, grecs_grecs_out )) {} } while (0)
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
/* %% [5.0] fread()/read() definition of YY_INPUT goes here unless we're doing C++ \ */\
	if ( YY_CURRENT_BUFFER_LVALUE->grecs_grecs__is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( grecs_grecs_in )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( grecs_grecs_in ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, grecs_grecs_in))==0 && ferror(grecs_grecs_in)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(grecs_grecs_in); \
			} \
		}\
\
/* %if-c++-only C++ definition \ */\
/* %endif */

#endif

/* No semi-colon after return; correct usage is to write "grecs_grecs_terminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef grecs_grecs_terminate
#define grecs_grecs_terminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
/* %if-c-only */
#define YY_FATAL_ERROR(msg) grecs_grecs__fatal_error( msg )
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

/* %if-tables-serialization structures and prototypes */
/* %not-for-header */

/* %ok-for-header */

/* %not-for-header */

/* %tables-grecs_grecs_dmap generated elements */
/* %endif */
/* end tables serialization structures and prototypes */

/* %ok-for-header */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1
/* %if-c-only Standard (non-C++) definition */

extern int grecs_grecs_lex (void);

#define YY_DECL int grecs_grecs_lex (void)
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after grecs_grecs_text and grecs_grecs_leng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK /*LINTED*/break;
#endif

/* %% [6.0] YY_RULE_SETUP definition goes here */
#define YY_RULE_SETUP \
	if ( grecs_grecs_leng > 0 ) \
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__at_bol = \
				(grecs_grecs_text[grecs_grecs_leng - 1] == '\n'); \
	YY_USER_ACTION

/* %not-for-header */

/** The main scanner function which does all the work.
 */
YY_DECL
{
	grecs_grecs__state_type grecs_grecs__current_state;
	char *grecs_grecs__cp, *grecs_grecs__bp;
	int grecs_grecs__act;
    
	if ( !(grecs_grecs__init) )
		{
		(grecs_grecs__init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (grecs_grecs__start) )
			(grecs_grecs__start) = 1;	/* first start state */

		if ( ! grecs_grecs_in )
/* %if-c-only */
			grecs_grecs_in = stdin;
/* %endif */
/* %if-c++-only */
/* %endif */

		if ( ! grecs_grecs_out )
/* %if-c-only */
			grecs_grecs_out = stdout;
/* %endif */
/* %if-c++-only */
/* %endif */

		if ( ! YY_CURRENT_BUFFER ) {
			grecs_grecs_ensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				grecs_grecs__create_buffer(grecs_grecs_in,YY_BUF_SIZE );
		}

		grecs_grecs__load_buffer_state( );
		}

	{
/* %% [7.0] user's declarations go here */
#line 169 "grecs-lex.l"

         /* C-style comments */
#line 1315 "grecs-lex.c"

	while ( /*CONSTCOND*/1 )		/* loops until end-of-file is reached */
		{
/* %% [8.0] grecs_grecs_more()-related code goes here */
		grecs_grecs__cp = (grecs_grecs__c_buf_p);

		/* Support of grecs_grecs_text. */
		*grecs_grecs__cp = (grecs_grecs__hold_char);

		/* grecs_grecs__bp points to the position in grecs_grecs__ch_buf of the start of
		 * the current run.
		 */
		grecs_grecs__bp = grecs_grecs__cp;

/* %% [9.0] code to set up and find next match goes here */
		grecs_grecs__current_state = (grecs_grecs__start);
		grecs_grecs__current_state += YY_AT_BOL();
grecs_grecs__match:
		do
			{
			YY_CHAR grecs_grecs__c = grecs_grecs__ec[YY_SC_TO_UI(*grecs_grecs__cp)] ;
			if ( grecs_grecs__accept[grecs_grecs__current_state] )
				{
				(grecs_grecs__last_accepting_state) = grecs_grecs__current_state;
				(grecs_grecs__last_accepting_cpos) = grecs_grecs__cp;
				}
			while ( grecs_grecs__chk[grecs_grecs__base[grecs_grecs__current_state] + grecs_grecs__c] != grecs_grecs__current_state )
				{
				grecs_grecs__current_state = (int) grecs_grecs__def[grecs_grecs__current_state];
				if ( grecs_grecs__current_state >= 258 )
					grecs_grecs__c = grecs_grecs__meta[(unsigned int) grecs_grecs__c];
				}
			grecs_grecs__current_state = grecs_grecs__nxt[grecs_grecs__base[grecs_grecs__current_state] + (unsigned int) grecs_grecs__c];
			++grecs_grecs__cp;
			}
		while ( grecs_grecs__base[grecs_grecs__current_state] != 919 );

grecs_grecs__find_action:
/* %% [10.0] code to find the action number goes here */
		grecs_grecs__act = grecs_grecs__accept[grecs_grecs__current_state];
		if ( grecs_grecs__act == 0 )
			{ /* have to back up */
			grecs_grecs__cp = (grecs_grecs__last_accepting_cpos);
			grecs_grecs__current_state = (grecs_grecs__last_accepting_state);
			grecs_grecs__act = grecs_grecs__accept[grecs_grecs__current_state];
			}

		YY_DO_BEFORE_ACTION;

/* %% [11.0] code for grecs_grecs_lineno update goes here */

do_action:	/* This label is used only to access EOF actions. */

/* %% [12.0] debug code goes here */
		if ( grecs_grecs__flex_debug )
			{
			if ( grecs_grecs__act == 0 )
				fprintf( stderr, "--scanner backing up\n" );
			else if ( grecs_grecs__act < 39 )
				fprintf( stderr, "--accepting rule at line %ld (\"%s\")\n",
				         (long)grecs_grecs__rule_linenum[grecs_grecs__act], grecs_grecs_text );
			else if ( grecs_grecs__act == 39 )
				fprintf( stderr, "--accepting default rule (\"%s\")\n",
				         grecs_grecs_text );
			else if ( grecs_grecs__act == 40 )
				fprintf( stderr, "--(end of buffer or a NUL)\n" );
			else
				fprintf( stderr, "--EOF (start condition %d)\n", YY_START );
			}

		switch ( grecs_grecs__act )
	{ /* beginning of action switch */
/* %% [13.0] actions go here */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*grecs_grecs__cp = (grecs_grecs__hold_char);
			grecs_grecs__cp = (grecs_grecs__last_accepting_cpos);
			grecs_grecs__current_state = (grecs_grecs__last_accepting_state);
			goto grecs_grecs__find_action;

case 1:
YY_RULE_SETUP
#line 171 "grecs-lex.l"
{ BEGIN(COMMENT); beginning_of_line = 0; }
	YY_BREAK

case 2:
YY_RULE_SETUP
#line 173 "grecs-lex.l"
/* eat anything that's not a '*' */
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 174 "grecs-lex.l"
/* eat up '*'s not followed by '/'s */
	YY_BREAK
case 4:
/* rule 4 can match eol */
YY_RULE_SETUP
#line 175 "grecs-lex.l"
{ grecs_locus_point_advance_line(grecs_current_locus_point);
	      emit_syncline = 1; }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 177 "grecs-lex.l"
{ BEGIN(initial_state); }
	YY_BREAK

/* Line directive */
case 6:
/* rule 6 can match eol */
YY_RULE_SETUP
#line 181 "grecs-lex.l"
{
		 grecs_parse_line_directive_cpp(grecs_grecs_text,
						&grecs_grecs_lloc,
						&grecs_current_locus_point);
	 }
	YY_BREAK
case 7:
/* rule 7 can match eol */
YY_RULE_SETUP
#line 186 "grecs-lex.l"
{
		 grecs_parse_line_directive(grecs_grecs_text,
					    &grecs_grecs_lloc,
					    &grecs_current_locus_point);
	 }
	YY_BREAK

/* Line directive */
case 8:
/* rule 8 can match eol */
YY_RULE_SETUP
#line 194 "grecs-lex.l"
{
		 grecs_parse_line_directive_cpp(grecs_grecs_text,
						&grecs_grecs_lloc,
						&grecs_current_locus_point);
		 ECHO;
	 }
	YY_BREAK
case 9:
/* rule 9 can match eol */
YY_RULE_SETUP
#line 200 "grecs-lex.l"
{
		 grecs_parse_line_directive(grecs_grecs_text,
					    &grecs_grecs_lloc,
					    &grecs_current_locus_point);
		 ECHO;
	 }
	YY_BREAK


/* Includes */
case 10:
/* rule 10 can match eol */
*grecs_grecs__cp = (grecs_grecs__hold_char); /* undo effects of setting up grecs_grecs_text */
YY_LINENO_REWIND_TO(grecs_grecs__cp - 1);
(grecs_grecs__c_buf_p) = grecs_grecs__cp -= 1;
YY_DO_BEFORE_ACTION; /* set up grecs_grecs_text again */
YY_RULE_SETUP
#line 210 "grecs-lex.l"
{ parse_include(grecs_grecs_text); }
	YY_BREAK
/* Error directives */
case 11:
/* rule 11 can match eol */
YY_RULE_SETUP
#line 212 "grecs-lex.l"
{
  grecs_error(&grecs_grecs_lloc, 0, "%s", report_diag(sizeof("error")));
 }
	YY_BREAK
case 12:
/* rule 12 can match eol */
YY_RULE_SETUP
#line 215 "grecs-lex.l"
{
  grecs_warning(&grecs_grecs_lloc, 0, "%s", report_diag(sizeof("warning")));
 }
	YY_BREAK
case 13:
/* rule 13 can match eol */
YY_RULE_SETUP
#line 218 "grecs-lex.l"
{
  grecs_error(&grecs_grecs_lloc, 0, "%s", report_diag(sizeof("abend")));
  ctx_abort();
  grecs_grecs_grecs_error_suppress = 1;
  return 0;
 }
	YY_BREAK
/* End-of-line comments */
case 14:
/* rule 14 can match eol */
YY_RULE_SETUP
#line 225 "grecs-lex.l"
{ grecs_locus_point_advance_line(grecs_current_locus_point);
            emit_syncline = 1; }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 227 "grecs-lex.l"
/* end-of-file comment */;
	YY_BREAK
case 16:
/* rule 16 can match eol */
YY_RULE_SETUP
#line 228 "grecs-lex.l"
{ grecs_locus_point_advance_line(grecs_current_locus_point);
            emit_syncline = 1; }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 230 "grecs-lex.l"
/* end-of-file comment */;
	YY_BREAK

/* Identifiers */
case 18:
YY_RULE_SETUP
#line 233 "grecs-lex.l"
return ident();
	YY_BREAK
/* Strings */
case 19:
YY_RULE_SETUP
#line 235 "grecs-lex.l"
{
	                   grecs_line_begin();
	                   grecs_line_add(grecs_grecs_text, grecs_grecs_leng);
                           grecs_grecs_lval.string = grecs_line_finish();
                           return STRING; }
	YY_BREAK
/* Quoted strings */
case 20:
YY_RULE_SETUP
#line 241 "grecs-lex.l"
{ grecs_line_begin();
                        grecs_line_add(grecs_grecs_text + 1, grecs_grecs_leng - 2);
                        grecs_grecs_lval.string = grecs_line_finish();
                        qstring_locus_fixup();
                        return qstring(); }
	YY_BREAK
case 21:
/* rule 21 can match eol */
YY_RULE_SETUP
#line 246 "grecs-lex.l"
{ BEGIN(STR);
                         grecs_line_begin();
		         grecs_line_acc_grow_unescape_last(grecs_grecs_text + 1,
                                                           grecs_grecs_leng - 1,
                                                           &grecs_grecs_lloc);
                         grecs_locus_point_advance_line(grecs_current_locus_point); }
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 252 "grecs-lex.l"
{ BEGIN(STR);
                         grecs_line_begin();
		         grecs_line_acc_grow_unescape_last(grecs_grecs_text + 1,
                                                           grecs_grecs_leng - 1, 
                                                           &grecs_grecs_lloc); }
	YY_BREAK

case 23:
/* rule 23 can match eol */
YY_RULE_SETUP
#line 258 "grecs-lex.l"
{ grecs_line_acc_grow_unescape_last(grecs_grecs_text, grecs_grecs_leng, &grecs_grecs_lloc);
                   grecs_locus_point_advance_line(grecs_current_locus_point); }
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 260 "grecs-lex.l"
{ grecs_line_acc_grow_unescape_last(grecs_grecs_text, grecs_grecs_leng, &grecs_grecs_lloc); }
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 261 "grecs-lex.l"
{ BEGIN(initial_state);
                   if (grecs_grecs_leng > 1) 
			grecs_line_add(grecs_grecs_text, grecs_grecs_leng - 1); 
                   grecs_grecs_lval.string = grecs_line_finish();
		   qstring_locus_fixup();    
		   return qstring(); }
	YY_BREAK

/* Multiline strings */
case 26:
/* rule 26 can match eol */
#line 270 "grecs-lex.l"
case 27:
/* rule 27 can match eol */
#line 271 "grecs-lex.l"
case 28:
/* rule 28 can match eol */
#line 272 "grecs-lex.l"
case 29:
/* rule 29 can match eol */
#line 273 "grecs-lex.l"
case 30:
/* rule 30 can match eol */
#line 274 "grecs-lex.l"
case 31:
/* rule 31 can match eol */
YY_RULE_SETUP
#line 274 "grecs-lex.l"
{
                BEGIN(ML);
		multiline_begin(grecs_grecs_text+2); }
	YY_BREAK

/* Ignore m4 line statements */
case 32:
/* rule 32 can match eol */
YY_RULE_SETUP
#line 279 "grecs-lex.l"
{
	grecs_locus_point_advance_line(grecs_current_locus_point);
}
	YY_BREAK
case 33:
/* rule 33 can match eol */
YY_RULE_SETUP
#line 282 "grecs-lex.l"
{ char *p = multiline_strip_tabs(grecs_grecs_text);
	   
       if (!strncmp(p, multiline_delimiter, multiline_delimiter_len)
           && isemptystr(p + multiline_delimiter_len - grecs_grecs_text)) {
	       grecs_free(multiline_delimiter);
	       multiline_delimiter = NULL;
	       BEGIN(initial_state);
	       grecs_grecs_lval.string = grecs_line_finish();

	       /* Update end pos */
	       grecs_grecs_lloc.end.line--;
	       for (grecs_grecs_lloc.end.col = 0,
			    p = grecs_grecs_lval.string + strlen(grecs_grecs_lval.string) - 1;
		    p > grecs_grecs_lval.string && p[-1] != '\n';
		    grecs_grecs_lloc.end.col++, p--);
	       if (grecs_grecs_lloc.end.col == 0)
		       grecs_grecs_lloc.end.col = 1;
	       return MSTRING;
       }
       grecs_locus_point_advance_line(grecs_current_locus_point);
       multiline_add(p);
    }
	YY_BREAK

case 34:
YY_RULE_SETUP
#line 306 "grecs-lex.l"
;
	YY_BREAK
/* Other tokens */
case 35:
/* rule 35 can match eol */
YY_RULE_SETUP
#line 308 "grecs-lex.l"
{ grecs_locus_point_advance_line(grecs_current_locus_point); } 
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 309 "grecs-lex.l"
return grecs_grecs_text[0];
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 310 "grecs-lex.l"
{ if (isascii(grecs_grecs_text[0]) && isprint(grecs_grecs_text[0]))
		grecs_error(&grecs_grecs_lloc, 0,
			     _("stray character %c"), grecs_grecs_text[0]);
           else
		grecs_error(&grecs_grecs_lloc, 0,
			     _("stray character \\%03o"),
			       (unsigned char) grecs_grecs_text[0]); }
	YY_BREAK

case 38:
/* rule 38 can match eol */
YY_RULE_SETUP
#line 319 "grecs-lex.l"
{  grecs_locus_point_advance_line(grecs_current_locus_point);
	    ECHO; }
	YY_BREAK

case 39:
YY_RULE_SETUP
#line 323 "grecs-lex.l"
ECHO;
	YY_BREAK
#line 1711 "grecs-lex.c"
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(COMMENT):
case YY_STATE_EOF(ML):
case YY_STATE_EOF(STR):
case YY_STATE_EOF(PP):
	grecs_grecs_terminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int grecs_grecs__amount_of_matched_text = (int) (grecs_grecs__cp - (grecs_grecs_text_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*grecs_grecs__cp = (grecs_grecs__hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed grecs_grecs_in at a new source and called
			 * grecs_grecs_lex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(grecs_grecs__n_chars) = YY_CURRENT_BUFFER_LVALUE->grecs_grecs__n_chars;
/* %if-c-only */
			YY_CURRENT_BUFFER_LVALUE->grecs_grecs__input_file = grecs_grecs_in;
/* %endif */
/* %if-c++-only */
/* %endif */
			YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for grecs_grecs__c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since grecs_grecs__c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (grecs_grecs__c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[(grecs_grecs__n_chars)] )
			{ /* This was really a NUL. */
			grecs_grecs__state_type grecs_grecs__next_state;

			(grecs_grecs__c_buf_p) = (grecs_grecs_text_ptr) + grecs_grecs__amount_of_matched_text;

			grecs_grecs__current_state = grecs_grecs__get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * grecs_grecs__get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			grecs_grecs__next_state = grecs_grecs__try_NUL_trans( grecs_grecs__current_state );

			grecs_grecs__bp = (grecs_grecs_text_ptr) + YY_MORE_ADJ;

			if ( grecs_grecs__next_state )
				{
				/* Consume the NUL. */
				grecs_grecs__cp = ++(grecs_grecs__c_buf_p);
				grecs_grecs__current_state = grecs_grecs__next_state;
				goto grecs_grecs__match;
				}

			else
				{
/* %% [14.0] code to do back-up for compressed tables and set up grecs_grecs__cp goes here */
				grecs_grecs__cp = (grecs_grecs__c_buf_p);
				goto grecs_grecs__find_action;
				}
			}

		else switch ( grecs_grecs__get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(grecs_grecs__did_buffer_switch_on_eof) = 0;

				if ( grecs_grecs_wrap( ) )
					{
					/* Note: because we've taken care in
					 * grecs_grecs__get_next_buffer() to have set up
					 * grecs_grecs_text, we can now set up
					 * grecs_grecs__c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(grecs_grecs__c_buf_p) = (grecs_grecs_text_ptr) + YY_MORE_ADJ;

					grecs_grecs__act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (grecs_grecs__did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(grecs_grecs__c_buf_p) =
					(grecs_grecs_text_ptr) + grecs_grecs__amount_of_matched_text;

				grecs_grecs__current_state = grecs_grecs__get_previous_state(  );

				grecs_grecs__cp = (grecs_grecs__c_buf_p);
				grecs_grecs__bp = (grecs_grecs_text_ptr) + YY_MORE_ADJ;
				goto grecs_grecs__match;

			case EOB_ACT_LAST_MATCH:
				(grecs_grecs__c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[(grecs_grecs__n_chars)];

				grecs_grecs__current_state = grecs_grecs__get_previous_state(  );

				grecs_grecs__cp = (grecs_grecs__c_buf_p);
				grecs_grecs__bp = (grecs_grecs_text_ptr) + YY_MORE_ADJ;
				goto grecs_grecs__find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of user's declarations */
} /* end of grecs_grecs_lex */
/* %ok-for-header */

/* %if-c++-only */
/* %not-for-header */

/* %ok-for-header */

/* %endif */

/* grecs_grecs__get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
/* %if-c-only */
static int grecs_grecs__get_next_buffer (void)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    	char *dest = YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf;
	char *source = (grecs_grecs_text_ptr);
	grecs_grecs__size_t number_to_move, i;
	int ret_val;

	if ( (grecs_grecs__c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[(grecs_grecs__n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->grecs_grecs__fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (grecs_grecs__c_buf_p) - (grecs_grecs_text_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (grecs_grecs__size_t) ((grecs_grecs__c_buf_p) - (grecs_grecs_text_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__n_chars = (grecs_grecs__n_chars) = 0;

	else
		{
			grecs_grecs__size_t num_to_read =
			YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER_LVALUE;

			int grecs_grecs__c_buf_p_offset =
				(int) ((grecs_grecs__c_buf_p) - b->grecs_grecs__ch_buf);

			if ( b->grecs_grecs__is_our_buffer )
				{
				grecs_grecs__size_t new_size = b->grecs_grecs__buf_size * 2;

				if ( new_size <= 0 )
					b->grecs_grecs__buf_size += b->grecs_grecs__buf_size / 8;
				else
					b->grecs_grecs__buf_size *= 2;

				b->grecs_grecs__ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					grecs_grecs_realloc((void *) b->grecs_grecs__ch_buf,b->grecs_grecs__buf_size + 2  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->grecs_grecs__ch_buf = 0;

			if ( ! b->grecs_grecs__ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(grecs_grecs__c_buf_p) = &b->grecs_grecs__ch_buf[grecs_grecs__c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[number_to_move]),
			(grecs_grecs__n_chars), num_to_read );

		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__n_chars = (grecs_grecs__n_chars);
		}

	if ( (grecs_grecs__n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			grecs_grecs_restart(grecs_grecs_in  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((grecs_grecs__size_t) ((grecs_grecs__n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		grecs_grecs__size_t new_size = (grecs_grecs__n_chars) + number_to_move + ((grecs_grecs__n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf = (char *) grecs_grecs_realloc((void *) YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf,new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs__get_next_buffer()" );
	}

	(grecs_grecs__n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[(grecs_grecs__n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[(grecs_grecs__n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(grecs_grecs_text_ptr) = &YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[0];

	return ret_val;
}

/* grecs_grecs__get_previous_state - get the state just before the EOB char was reached */

/* %if-c-only */
/* %not-for-header */

    static grecs_grecs__state_type grecs_grecs__get_previous_state (void)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	grecs_grecs__state_type grecs_grecs__current_state;
	char *grecs_grecs__cp;
    
/* %% [15.0] code to get the start state into grecs_grecs__current_state goes here */
	grecs_grecs__current_state = (grecs_grecs__start);
	grecs_grecs__current_state += YY_AT_BOL();

	for ( grecs_grecs__cp = (grecs_grecs_text_ptr) + YY_MORE_ADJ; grecs_grecs__cp < (grecs_grecs__c_buf_p); ++grecs_grecs__cp )
		{
/* %% [16.0] code to find the next state goes here */
		YY_CHAR grecs_grecs__c = (*grecs_grecs__cp ? grecs_grecs__ec[YY_SC_TO_UI(*grecs_grecs__cp)] : 1);
		if ( grecs_grecs__accept[grecs_grecs__current_state] )
			{
			(grecs_grecs__last_accepting_state) = grecs_grecs__current_state;
			(grecs_grecs__last_accepting_cpos) = grecs_grecs__cp;
			}
		while ( grecs_grecs__chk[grecs_grecs__base[grecs_grecs__current_state] + grecs_grecs__c] != grecs_grecs__current_state )
			{
			grecs_grecs__current_state = (int) grecs_grecs__def[grecs_grecs__current_state];
			if ( grecs_grecs__current_state >= 258 )
				grecs_grecs__c = grecs_grecs__meta[(unsigned int) grecs_grecs__c];
			}
		grecs_grecs__current_state = grecs_grecs__nxt[grecs_grecs__base[grecs_grecs__current_state] + (unsigned int) grecs_grecs__c];
		}

	return grecs_grecs__current_state;
}

/* grecs_grecs__try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = grecs_grecs__try_NUL_trans( current_state );
 */
/* %if-c-only */
    static grecs_grecs__state_type grecs_grecs__try_NUL_trans  (grecs_grecs__state_type grecs_grecs__current_state )
/* %endif */
/* %if-c++-only */
/* %endif */
{
	int grecs_grecs__is_jam;
    /* %% [17.0] code to find the next state, and perhaps do backing up, goes here */
	char *grecs_grecs__cp = (grecs_grecs__c_buf_p);

	YY_CHAR grecs_grecs__c = 1;
	if ( grecs_grecs__accept[grecs_grecs__current_state] )
		{
		(grecs_grecs__last_accepting_state) = grecs_grecs__current_state;
		(grecs_grecs__last_accepting_cpos) = grecs_grecs__cp;
		}
	while ( grecs_grecs__chk[grecs_grecs__base[grecs_grecs__current_state] + grecs_grecs__c] != grecs_grecs__current_state )
		{
		grecs_grecs__current_state = (int) grecs_grecs__def[grecs_grecs__current_state];
		if ( grecs_grecs__current_state >= 258 )
			grecs_grecs__c = grecs_grecs__meta[(unsigned int) grecs_grecs__c];
		}
	grecs_grecs__current_state = grecs_grecs__nxt[grecs_grecs__base[grecs_grecs__current_state] + (unsigned int) grecs_grecs__c];
	grecs_grecs__is_jam = (grecs_grecs__current_state == 257);

		return grecs_grecs__is_jam ? 0 : grecs_grecs__current_state;
}

#ifndef YY_NO_UNPUT
/* %if-c-only */

/* %endif */
#endif

/* %if-c-only */
#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int grecs_grecs_input (void)
#else
    static int input  (void)
#endif

/* %endif */
/* %if-c++-only */
/* %endif */
{
	int c;
    
	*(grecs_grecs__c_buf_p) = (grecs_grecs__hold_char);

	if ( *(grecs_grecs__c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* grecs_grecs__c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (grecs_grecs__c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[(grecs_grecs__n_chars)] )
			/* This was really a NUL. */
			*(grecs_grecs__c_buf_p) = '\0';

		else
			{ /* need more input */
			grecs_grecs__size_t offset = (grecs_grecs__c_buf_p) - (grecs_grecs_text_ptr);
			++(grecs_grecs__c_buf_p);

			switch ( grecs_grecs__get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because grecs_grecs__g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					grecs_grecs_restart(grecs_grecs_in );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( grecs_grecs_wrap( ) )
						return EOF;

					if ( ! (grecs_grecs__did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return grecs_grecs_input();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(grecs_grecs__c_buf_p) = (grecs_grecs_text_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (grecs_grecs__c_buf_p);	/* cast for 8-bit char's */
	*(grecs_grecs__c_buf_p) = '\0';	/* preserve grecs_grecs_text */
	(grecs_grecs__hold_char) = *++(grecs_grecs__c_buf_p);

/* %% [19.0] update BOL and grecs_grecs_lineno */
	YY_CURRENT_BUFFER_LVALUE->grecs_grecs__at_bol = (c == '\n');

	return c;
}
/* %if-c-only */
#endif	/* ifndef YY_NO_INPUT */
/* %endif */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
/* %if-c-only */
    void grecs_grecs_restart  (FILE * input_file )
/* %endif */
/* %if-c++-only */
/* %endif */
{
    
	if ( ! YY_CURRENT_BUFFER ){
        grecs_grecs_ensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            grecs_grecs__create_buffer(grecs_grecs_in,YY_BUF_SIZE );
	}

	grecs_grecs__init_buffer(YY_CURRENT_BUFFER,input_file );
	grecs_grecs__load_buffer_state( );
}

/* %if-c++-only */
/* %endif */

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
/* %if-c-only */
    void grecs_grecs__switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
/* %endif */
/* %if-c++-only */
/* %endif */
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		grecs_grecs_pop_buffer_state();
	 *		grecs_grecs_push_buffer_state(new_buffer);
     */
	grecs_grecs_ensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(grecs_grecs__c_buf_p) = (grecs_grecs__hold_char);
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buf_pos = (grecs_grecs__c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__n_chars = (grecs_grecs__n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	grecs_grecs__load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (grecs_grecs_wrap()) processing, but the only time this flag
	 * is looked at is after grecs_grecs_wrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(grecs_grecs__did_buffer_switch_on_eof) = 1;
}

/* %if-c-only */
static void grecs_grecs__load_buffer_state  (void)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    	(grecs_grecs__n_chars) = YY_CURRENT_BUFFER_LVALUE->grecs_grecs__n_chars;
	(grecs_grecs_text_ptr) = (grecs_grecs__c_buf_p) = YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buf_pos;
/* %if-c-only */
	grecs_grecs_in = YY_CURRENT_BUFFER_LVALUE->grecs_grecs__input_file;
/* %endif */
/* %if-c++-only */
/* %endif */
	(grecs_grecs__hold_char) = *(grecs_grecs__c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
/* %if-c-only */
    YY_BUFFER_STATE grecs_grecs__create_buffer  (FILE * file, int  size )
/* %endif */
/* %if-c++-only */
/* %endif */
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) grecs_grecs_alloc(sizeof( struct grecs_grecs__buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs__create_buffer()" );

	b->grecs_grecs__buf_size = (grecs_grecs__size_t)size;

	/* grecs_grecs__ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->grecs_grecs__ch_buf = (char *) grecs_grecs_alloc(b->grecs_grecs__buf_size + 2  );
	if ( ! b->grecs_grecs__ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs__create_buffer()" );

	b->grecs_grecs__is_our_buffer = 1;

	grecs_grecs__init_buffer(b,file );

	return b;
}

/* %if-c++-only */
/* %endif */

/** Destroy the buffer.
 * @param b a buffer created with grecs_grecs__create_buffer()
 * 
 */
/* %if-c-only */
    void grecs_grecs__delete_buffer (YY_BUFFER_STATE  b )
/* %endif */
/* %if-c++-only */
/* %endif */
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->grecs_grecs__is_our_buffer )
		grecs_grecs_free((void *) b->grecs_grecs__ch_buf  );

	grecs_grecs_free((void *) b  );
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a grecs_grecs_restart() or at EOF.
 */
/* %if-c-only */
    static void grecs_grecs__init_buffer  (YY_BUFFER_STATE  b, FILE * file )
/* %endif */
/* %if-c++-only */
/* %endif */

{
	int oerrno = errno;
    
	grecs_grecs__flush_buffer(b );

/* %if-c-only */
	b->grecs_grecs__input_file = file;
/* %endif */
/* %if-c++-only */
/* %endif */
	b->grecs_grecs__fill_buffer = 1;

    /* If b is the current buffer, then grecs_grecs__init_buffer was _probably_
     * called from grecs_grecs_restart() or through grecs_grecs__get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->grecs_grecs__bs_lineno = 1;
        b->grecs_grecs__bs_column = 0;
    }

/* %if-c-only */

        b->grecs_grecs__is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
/* %endif */
/* %if-c++-only */
/* %endif */
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
/* %if-c-only */
    void grecs_grecs__flush_buffer (YY_BUFFER_STATE  b )
/* %endif */
/* %if-c++-only */
/* %endif */
{
    	if ( ! b )
		return;

	b->grecs_grecs__n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->grecs_grecs__ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->grecs_grecs__ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->grecs_grecs__buf_pos = &b->grecs_grecs__ch_buf[0];

	b->grecs_grecs__at_bol = 1;
	b->grecs_grecs__buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		grecs_grecs__load_buffer_state( );
}

/* %if-c-or-c++ */
/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
/* %if-c-only */
void grecs_grecs_push_buffer_state (YY_BUFFER_STATE new_buffer )
/* %endif */
/* %if-c++-only */
/* %endif */
{
    	if (new_buffer == NULL)
		return;

	grecs_grecs_ensure_buffer_stack();

	/* This block is copied from grecs_grecs__switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(grecs_grecs__c_buf_p) = (grecs_grecs__hold_char);
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buf_pos = (grecs_grecs__c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__n_chars = (grecs_grecs__n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(grecs_grecs__buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from grecs_grecs__switch_to_buffer. */
	grecs_grecs__load_buffer_state( );
	(grecs_grecs__did_buffer_switch_on_eof) = 1;
}
/* %endif */

/* %if-c-or-c++ */
/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
/* %if-c-only */
void grecs_grecs_pop_buffer_state (void)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    	if (!YY_CURRENT_BUFFER)
		return;

	grecs_grecs__delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((grecs_grecs__buffer_stack_top) > 0)
		--(grecs_grecs__buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		grecs_grecs__load_buffer_state( );
		(grecs_grecs__did_buffer_switch_on_eof) = 1;
	}
}
/* %endif */

/* %if-c-or-c++ */
/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
/* %if-c-only */
static void grecs_grecs_ensure_buffer_stack (void)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	grecs_grecs__size_t num_to_alloc;
    
	if (!(grecs_grecs__buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1; // After all that talk, this was set to 1 anyways...
		(grecs_grecs__buffer_stack) = (struct grecs_grecs__buffer_state**)grecs_grecs_alloc
								(num_to_alloc * sizeof(struct grecs_grecs__buffer_state*)
								);
		if ( ! (grecs_grecs__buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs_ensure_buffer_stack()" );
								  
		memset((grecs_grecs__buffer_stack), 0, num_to_alloc * sizeof(struct grecs_grecs__buffer_state*));
				
		(grecs_grecs__buffer_stack_max) = num_to_alloc;
		(grecs_grecs__buffer_stack_top) = 0;
		return;
	}

	if ((grecs_grecs__buffer_stack_top) >= ((grecs_grecs__buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		grecs_grecs__size_t grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (grecs_grecs__buffer_stack_max) + grow_size;
		(grecs_grecs__buffer_stack) = (struct grecs_grecs__buffer_state**)grecs_grecs_realloc
								((grecs_grecs__buffer_stack),
								num_to_alloc * sizeof(struct grecs_grecs__buffer_state*)
								);
		if ( ! (grecs_grecs__buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs_ensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((grecs_grecs__buffer_stack) + (grecs_grecs__buffer_stack_max), 0, grow_size * sizeof(struct grecs_grecs__buffer_state*));
		(grecs_grecs__buffer_stack_max) = num_to_alloc;
	}
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE grecs_grecs__scan_buffer  (char * base, grecs_grecs__size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) grecs_grecs_alloc(sizeof( struct grecs_grecs__buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs__scan_buffer()" );

	b->grecs_grecs__buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->grecs_grecs__buf_pos = b->grecs_grecs__ch_buf = base;
	b->grecs_grecs__is_our_buffer = 0;
	b->grecs_grecs__input_file = 0;
	b->grecs_grecs__n_chars = b->grecs_grecs__buf_size;
	b->grecs_grecs__is_interactive = 0;
	b->grecs_grecs__at_bol = 1;
	b->grecs_grecs__fill_buffer = 0;
	b->grecs_grecs__buffer_status = YY_BUFFER_NEW;

	grecs_grecs__switch_to_buffer(b  );

	return b;
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan a string. The next call to grecs_grecs_lex() will
 * scan from a @e copy of @a str.
 * @param grecs_grecs_str a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       grecs_grecs__scan_bytes() instead.
 */
YY_BUFFER_STATE grecs_grecs__scan_string (grecs_grecs_const char * grecs_grecs_str )
{
    
	return grecs_grecs__scan_bytes(grecs_grecs_str,strlen(grecs_grecs_str) );
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan the given bytes. The next call to grecs_grecs_lex() will
 * scan from a @e copy of @a bytes.
 * @param grecs_grecs_bytes the byte buffer to scan
 * @param _grecs_grecs_bytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE grecs_grecs__scan_bytes  (grecs_grecs_const char * grecs_grecs_bytes, grecs_grecs__size_t  _grecs_grecs_bytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	grecs_grecs__size_t n;
	grecs_grecs__size_t i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _grecs_grecs_bytes_len + 2;
	buf = (char *) grecs_grecs_alloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs__scan_bytes()" );

	for ( i = 0; i < _grecs_grecs_bytes_len; ++i )
		buf[i] = grecs_grecs_bytes[i];

	buf[_grecs_grecs_bytes_len] = buf[_grecs_grecs_bytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = grecs_grecs__scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in grecs_grecs__scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->grecs_grecs__is_our_buffer = 1;

	return b;
}
/* %endif */

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

/* %if-c-only */
static void grecs_grecs__fatal_error (grecs_grecs_const char* msg )
{
			(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}
/* %endif */
/* %if-c++-only */
/* %endif */

/* Redefine grecs_grecs_less() so it works in section 3 code. */

#undef grecs_grecs_less
#define grecs_grecs_less(n) \
	do \
		{ \
		/* Undo effects of setting up grecs_grecs_text. */ \
        int grecs_grecs_less_macro_arg = (n); \
        YY_LESS_LINENO(grecs_grecs_less_macro_arg);\
		grecs_grecs_text[grecs_grecs_leng] = (grecs_grecs__hold_char); \
		(grecs_grecs__c_buf_p) = grecs_grecs_text + grecs_grecs_less_macro_arg; \
		(grecs_grecs__hold_char) = *(grecs_grecs__c_buf_p); \
		*(grecs_grecs__c_buf_p) = '\0'; \
		grecs_grecs_leng = grecs_grecs_less_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/* %if-c-only */
/* %if-reentrant */
/* %endif */

/** Get the current line number.
 * 
 */
int grecs_grecs_get_lineno  (void)
{
        
    return grecs_grecs_lineno;
}

/** Get the input stream.
 * 
 */
FILE *grecs_grecs_get_in  (void)
{
        return grecs_grecs_in;
}

/** Get the output stream.
 * 
 */
FILE *grecs_grecs_get_out  (void)
{
        return grecs_grecs_out;
}

/** Get the length of the current token.
 * 
 */
grecs_grecs__size_t grecs_grecs_get_leng  (void)
{
        return grecs_grecs_leng;
}

/** Get the current token.
 * 
 */

char *grecs_grecs_get_text  (void)
{
        return grecs_grecs_text;
}

/* %if-reentrant */
/* %endif */

/** Set the current line number.
 * @param _line_number line number
 * 
 */
void grecs_grecs_set_lineno (int  _line_number )
{
    
    grecs_grecs_lineno = _line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param _in_str A readable stream.
 * 
 * @see grecs_grecs__switch_to_buffer
 */
void grecs_grecs_set_in (FILE *  _in_str )
{
        grecs_grecs_in = _in_str ;
}

void grecs_grecs_set_out (FILE *  _out_str )
{
        grecs_grecs_out = _out_str ;
}

int grecs_grecs_get_debug  (void)
{
        return grecs_grecs__flex_debug;
}

void grecs_grecs_set_debug (int  _bdebug )
{
        grecs_grecs__flex_debug = _bdebug ;
}

/* %endif */

/* %if-reentrant */
/* %if-bison-bridge */
/* %endif */
/* %endif if-c-only */

/* %if-c-only */
static int grecs_grecs__init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from grecs_grecs_lex_destroy(), so don't allocate here.
     */

    (grecs_grecs__buffer_stack) = 0;
    (grecs_grecs__buffer_stack_top) = 0;
    (grecs_grecs__buffer_stack_max) = 0;
    (grecs_grecs__c_buf_p) = (char *) 0;
    (grecs_grecs__init) = 0;
    (grecs_grecs__start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    grecs_grecs_in = stdin;
    grecs_grecs_out = stdout;
#else
    grecs_grecs_in = (FILE *) 0;
    grecs_grecs_out = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * grecs_grecs_lex_init()
     */
    return 0;
}
/* %endif */

/* %if-c-only SNIP! this currently causes conflicts with the c++ scanner */
/* grecs_grecs_lex_destroy is for both reentrant and non-reentrant scanners. */
int grecs_grecs_lex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		grecs_grecs__delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		grecs_grecs_pop_buffer_state();
	}

	/* Destroy the stack itself. */
	grecs_grecs_free((grecs_grecs__buffer_stack) );
	(grecs_grecs__buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * grecs_grecs_lex() is called, initialization will occur. */
    grecs_grecs__init_globals( );

/* %if-reentrant */
/* %endif */
    return 0;
}
/* %endif */

/*
 * Internal utility routines.
 */

#ifndef grecs_grecs_text_ptr
static void grecs_grecs__flex_strncpy (char* s1, grecs_grecs_const char * s2, int n )
{
		
	int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int grecs_grecs__flex_strlen (grecs_grecs_const char * s )
{
	int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *grecs_grecs_alloc (grecs_grecs__size_t  size )
{
			return (void *) malloc( size );
}

void *grecs_grecs_realloc  (void * ptr, grecs_grecs__size_t  size )
{
		
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void grecs_grecs_free (void * ptr )
{
			free( (char *) ptr );	/* see grecs_grecs_realloc() for (char *) cast */
}

/* %if-tables-serialization definitions */
/* %define-grecs_grecs_tables   The name for this specific scanner's tables. */
#define YYTABLES_NAME "grecs_grecs_tables"
/* %endif */

/* %ok-for-header */

#line 323 "grecs-lex.l"


/*
 * Source inclusion support.
 */
static glob_t include_glob;
static size_t include_pos;
static int include_once;

static int
isglob(const char *s)
{
	return s[strcspn(s, "*?[")] != 0;
}

static int
parse_include(char *text)
{
	char *p;
	int rc = 1;
	int once;

	p = strchr(text, '#');
	p += strspn(p, " \t") + sizeof("include");
	if (strncmp(p, "_once", 5) == 0) {
		once = 1;
		p += 5;
	} else
		once = 0;
	p += strspn(p, " \t");
	
	if (!*p) {
		grecs_error(&grecs_grecs_lloc, 0, _("invalid include statement"));
		return 1;
	} else {
		size_t len;
		int allow_cwd;
		
		len = strlen (p);
		
		if (p[0] == '<' && p[len - 1] == '>') {
			allow_cwd = 0;
			p[len - 1] = 0;
			p++;
		} else if (p[0] == '"' && p[len - 1] == '"') {
			allow_cwd = 1;
			p[len - 1] = 0;
			p++;
		} else {
			grecs_error(&grecs_grecs_lloc, 0, _("invalid include statement"));
			return 1;
		}
		
		if (isglob(p)) {
			switch (glob(p, 0, NULL, &include_glob)) {
			case 0:
				include_pos = 0;
				include_once = once;
				break;
			case GLOB_NOSPACE:
				grecs_alloc_die();
			case GLOB_NOMATCH:
				break;
			default:
				grecs_error(&grecs_grecs_lloc, 0,  _("read error"));
			}
			p = NULL;
		} else if (p[0] != '/') {
			char *q = p;
			p = grecs_find_include_file(q, allow_cwd);
			if (!p)
				grecs_error(&grecs_grecs_lloc, 0,
					    _("%s: No such file or directory"),
					    q);
		}
 	}
  
	if (p)
		rc = push_source(p, once);
	else if (include_pos < include_glob.gl_pathc)
		rc = push_source(include_glob.gl_pathv[include_pos++], once);
	return rc;
}

/*
 * Include search path.
 */
static struct grecs_list *grecs_usr_include_path;
static struct grecs_list *grecs_std_include_path;

size_t
grecs_include_path_count(int flag)
{
    size_t count = 0;
    if (flag & GRECS_STD_INCLUDE)
	count += grecs_list_size(grecs_std_include_path);
    if (flag & GRECS_USR_INCLUDE)
	count += grecs_list_size(grecs_usr_include_path);
    return count;
}

static int
foreach_dir(struct grecs_list *list, int flag,
	    int (*fun)(int, const char *, void *), void *data)
{
    int rc = 0;
    struct grecs_list_entry *ep;

    for (ep = list->head; rc == 0 && ep; ep = ep->next)
	rc = fun(flag, ep->data, data);
    return rc;
}

int
grecs_foreach_include_dir(int flag, int (*fun)(int, const char *, void *),
			  void *data)
{
    int rc = 0;

    if (flag & GRECS_STD_INCLUDE)
	rc = foreach_dir(grecs_std_include_path, GRECS_STD_INCLUDE, fun, data);
    if (rc == 0 && (flag & GRECS_USR_INCLUDE))
	rc = foreach_dir(grecs_usr_include_path, GRECS_USR_INCLUDE, fun, data);
    return rc;
}

void
grecs_include_path_clear()
{
	if (grecs_usr_include_path)
		grecs_list_clear(grecs_usr_include_path);
	if (grecs_std_include_path)
		grecs_list_clear(grecs_std_include_path);
}

static void
incl_free(void *data)
{
	grecs_free(data);
}

void
grecs_include_path_setup_v(char **dirs)
{
	if (!grecs_usr_include_path) {
		grecs_usr_include_path = grecs_list_create();
		grecs_usr_include_path->free_entry = incl_free;
	}
	grecs_std_include_path = grecs_list_create();
	grecs_std_include_path->free_entry = incl_free;
	if (dirs) {
		int i;
		for (i = 0; dirs[i]; i++)
			/* FIXME: Element never freed */
			grecs_list_append(grecs_std_include_path,
					  grecs_strdup(dirs[i]));
	}
}

void
grecs_include_path_setup(const char *dir, ...)
{
	const char *p;
	char **argv = NULL;
	size_t argc = 0;
	size_t argi = 0;
	va_list ap;
	
	va_start(ap, dir);
	p = dir;
	while (1) {
		if (argi == argc) {
			if (argc == 0)
				argc = 16;
			else
				argc += 16;
			argv = grecs_realloc(argv, argc * sizeof(argv[0]));
		}
		argv[argi++] = (char*) p;
		if (!p)
			break;
		p = va_arg(ap, const char*);
	}
	grecs_include_path_setup_v(argv);
	grecs_free(argv);
	va_end(ap);
}

void
grecs_preproc_add_include_dir(char *dir)
{
	if (!grecs_usr_include_path) {
		grecs_usr_include_path = grecs_list_create();
		grecs_usr_include_path->free_entry = incl_free;
	}
	grecs_list_append(grecs_usr_include_path, grecs_strdup(dir));
}

struct file_data {
	const char *name;
	size_t namelen;
	char *buf;
	size_t buflen;
	int found;
};

static int
pp_list_find(struct grecs_list *list, struct file_data *dptr)
{
	struct grecs_list_entry *ep;

	if (!list)
		return 0;
	for (ep = list->head; !dptr->found && ep; ep = ep->next) {
		const char *dir = ep->data;
		size_t size = strlen (dir) + 1 + dptr->namelen + 1;
		if (size > dptr->buflen) {
			dptr->buflen = size;
			dptr->buf = grecs_realloc(dptr->buf, dptr->buflen);
		}
		strcpy(dptr->buf, dir);
		strcat(dptr->buf, "/");
		strcat(dptr->buf, dptr->name);
		dptr->found = access(dptr->buf, F_OK) == 0;
	}
	return dptr->found;
}

char *
grecs_find_include_file(const char *name, int allow_cwd)
{
	static char *cwd = ".";
	struct file_data fd;

	fd.name = name;
	fd.namelen = strlen(name);
	fd.buf = NULL;
	fd.buflen = 0;
	fd.found = 0;

	if (!grecs_usr_include_path)
		grecs_include_path_setup(NULL);
	if (allow_cwd) {
		grecs_list_append(grecs_usr_include_path, cwd);
		pp_list_find(grecs_usr_include_path, &fd);
		grecs_list_remove_tail(grecs_usr_include_path);
	} else
		pp_list_find(grecs_usr_include_path, &fd);

	if (!fd.found) {
		pp_list_find(grecs_std_include_path, &fd);
		if (!fd.found)
			return NULL;
	}
	return fd.buf;
}

/*
 * Input context support.
 */

struct input_file_ident {
	ino_t i_node;
	dev_t device;
	pid_t pid;
};

struct buffer_ctx {
	struct buffer_ctx *prev;	/* Pointer to previous context */
	struct grecs_locus locus;
	struct grecs_locus_point point;	/* Current input location */
	size_t namelen;		/* Length of the file name */
	struct input_file_ident id;
	FILE *infile;
	YY_BUFFER_STATE state;
};

static struct buffer_ctx *context_stack;
static struct input_file_ident input_file_ident;

#define STAT_ID_EQ(st,id) ((id).i_node == (st).st_ino       \
			    && (id).device == (st).st_dev)

static struct buffer_ctx *
ctx_lookup(struct stat *st)
{
	struct buffer_ctx *ctx;

	if (!context_stack)
		return NULL;

	for (ctx = context_stack; ctx; ctx = ctx->prev)
		if (STAT_ID_EQ(*st, ctx->id))
			break;
	return ctx;
}

static void
ctx_abort(void)
{
	/* Delete current input buffer */
	grecs_grecs__delete_buffer(YY_CURRENT_BUFFER);
			
	/* Clean-up context stack */
	while (context_stack) {
		struct buffer_ctx *prev = context_stack->prev;
		if (context_stack->id.pid > 0)
			kill(context_stack->id.pid, SIGKILL);
		fclose(context_stack->infile);
		grecs_free(context_stack);
		context_stack = prev;
	}
}

static struct grecs_symtab *incl_sources;

/* Calculate the hash of a struct input_file_ident.  */
static unsigned
incl_hasher(void *data, unsigned long n_buckets)
{
	const struct input_file_ident *id = data;
	return (id->i_node + id->device) % n_buckets;
}

/* Compare two input_file_idents for equality.  */
static int
incl_compare(void const *data1, void const *data2)
{
	const struct input_file_ident *id1 = data1;
	const struct input_file_ident *id2 = data2;
	return !(id1->device == id2->device && id1->i_node == id2->i_node);
}

static int
incl_copy(void *dst, void *src)
{
	memcpy(dst, src, sizeof(struct input_file_ident));
	return 0;
}

static int
source_lookup(struct stat *st)
{
	struct input_file_ident key;
	int install = 1;
  
	if (!incl_sources) {
		incl_sources = grecs_symtab_create(
			sizeof(struct input_file_ident), 
			incl_hasher,
			incl_compare,
			incl_copy,
			NULL,/*FIXME: alloc*/
			NULL);
		if (!incl_sources)
			grecs_alloc_die();
	}
	
	key.i_node = st->st_ino;
	key.device = st->st_dev;
	if (!grecs_symtab_lookup_or_install(incl_sources, &key, &install))
		grecs_alloc_die();
	return !install;
}

static int
push_source(const char *name, int once)
{
	FILE *fp;
	pid_t pid;
	struct buffer_ctx *ctx;
	struct stat st;
	int rc = stat(name, &st);

	if (context_stack) {
		if (rc) {
			grecs_error(&grecs_grecs_lloc, errno,
				    _("Cannot stat `%s'"), name);
			return 1;
		}

		if (grecs_current_locus_point.col &&
		    STAT_ID_EQ(st, input_file_ident)) {
			grecs_error(&grecs_grecs_lloc, 0, _("Recursive inclusion"));
			return 1;
		}
		
		if ((ctx = ctx_lookup(&st))) {
			grecs_error(&grecs_grecs_lloc, 0, _("Recursive inclusion"));
			if (ctx->prev)
				grecs_error(&ctx->prev->locus, 0,
					    _("`%s' already included here"),
					    name);
			else
				grecs_error(&grecs_grecs_lloc, 0,
					    _("`%s' already included at top level"),
					    name);
			return 1;
		}
	} else if (rc) {
		grecs_error(NULL, errno, _("Cannot stat `%s'"), name);
		return 1;
	}

	if (once && source_lookup(&st))
		return -1;

	if (grecs_preprocessor) {
		if (access(name,R_OK)) {
			grecs_error(context_stack ? &grecs_grecs_lloc : NULL, errno,
				    _("Cannot open `%s'"), name);
			return -1;
		}
		fp = grecs_preproc_extrn_start(name, &pid);
	} else {
		fp = fopen(name, "r");
		pid = -1;
	}
	if (!fp) {
		grecs_error(context_stack ? &grecs_grecs_lloc : NULL, errno,
			    _("Cannot open `%s'"), name);
		return 1;
	}

	/* Push current context */
	if (grecs_grecs_in) {
		ctx = grecs_malloc(sizeof(*ctx));
		ctx->locus = grecs_grecs_lloc;
		ctx->point = grecs_current_locus_point;
		ctx->id = input_file_ident;
		ctx->infile = grecs_grecs_in;
		ctx->state = YY_CURRENT_BUFFER;
		ctx->prev = context_stack;
		context_stack = ctx;
	}
	
	grecs_grecs_in = fp;
	grecs_grecs__switch_to_buffer(grecs_grecs__create_buffer(grecs_grecs_in,YY_BUF_SIZE));

	grecs_current_locus_point.file = grecs_install_text(name);
	grecs_current_locus_point.line = 1;
	grecs_current_locus_point.col = 0;
	
	input_file_ident.i_node = st.st_ino;
	input_file_ident.device = st.st_dev;
	input_file_ident.pid = pid;

	beginning_of_line = 1;
	
	if (grecs_grecs__flex_debug)
	  fprintf (stderr, "Processing file `%s'\n", name);

	if (initial_state == PP)
		emit_syncline = 1;
	
	return 0;
}

static int
pop_source(void)
{
	struct buffer_ctx *ctx;

	fclose(grecs_grecs_in);
	if (input_file_ident.pid > 0) {
		if (grecs_preproc_extrn_shutdown(input_file_ident.pid)) {
			/* Abort further input */
			ctx_abort();

			/* Suppress further errors */
			grecs_grecs_grecs_error_suppress = 1;

			/* Indicate there is no more input */
			return 1;
		}
	}
	
	if (!context_stack) {
		if (grecs_grecs__flex_debug)
			fprintf(stderr, "End of input\n");
		grecs_grecs_in = NULL;
		return 1;
	}
	
	/* Restore previous context */
	grecs_grecs_in = context_stack->infile;
	grecs_current_locus_point = context_stack->point;
	input_file_ident = context_stack->id;
	
	grecs_grecs__delete_buffer(YY_CURRENT_BUFFER);
	grecs_grecs__switch_to_buffer(context_stack->state);
	
	ctx = context_stack->prev;
	grecs_free(context_stack);
	context_stack = ctx;

	while (include_pos < include_glob.gl_pathc) {
		if (push_source(include_glob.gl_pathv[include_pos++],
				include_once) == 0)
			return 0;
	}

	if (include_glob.gl_pathc) {
		globfree(&include_glob);
		include_pos = include_glob.gl_pathc = 0;
	}
	
	if (grecs_grecs__flex_debug)
		fprintf(stderr, "Resuming file `%s' at line %lu\n",
			grecs_current_locus_point.file,
			(unsigned long) grecs_current_locus_point.line);

	if (initial_state == PP) {
		input();
		grecs_locus_point_advance_line(grecs_current_locus_point);
		emit_syncline = 1;
	}
	
	return 0;
}

/*
 * Internal lexer functions.
 */
int
grecs_grecs_wrap()
{
	return pop_source();
}

int
grecs_lex_begin(const char *name, int trace)
{
	grecs_grecs__flex_debug = trace;
	initial_state = INITIAL;
	grecs_line_acc_create();
	return push_source(name, 1);
}

void
grecs_lex_end(int err)
{
	grecs_line_acc_free();
}

/*
 * Here-document functions.
 */
static int
isemptystr(int off)
{
	for (; grecs_grecs_text[off] && isspace(grecs_grecs_text[off]); off++)
		;
	if (grecs_grecs_text[off] == ';') {
		int i;
		for (i = off + 1; grecs_grecs_text[i]; i++) 
			if (!isspace(grecs_grecs_text[i]))
				return 0;
		grecs_grecs_less(off);
		return 1;
	}
	return grecs_grecs_text[off] == 0;
}

char *
multiline_strip_tabs(char *text)
{
	if (char_to_strip)
		for (; *text && char_to_strip(*text); text++)
			;
	return text;
}

static void
multiline_add(char *s)
{
	if (multiline_unescape) {
		for (; *s; s++)	{
			if (*s == '\\') {
				grecs_line_acc_grow_char_unescape(s[1]);
				++s;
			} else
				grecs_line_acc_grow_char(*s);
		}
	} else
		grecs_line_add(s, strlen(s));
}

static int
is_tab(char c)
{
	return c == '\t';
}
 
static int
is_ws(char c)
{
	return c == '\t' || c == ' ';
}

void
multiline_begin(char *p)
{
	if (*p == '-') {
		if (*++p == ' ') {
			char_to_strip = is_ws;
			p++;
		} else
			char_to_strip = is_tab;
	} else
		char_to_strip = NULL;
	if (*p == '\\') {
		p++;
		multiline_unescape = 0;
		multiline_delimiter_len = strcspn(p, " \t");
	} else if (*p == '"') {
		char *q;
 
		p++;
		multiline_unescape = 0;
		q = strchr(p, '"');
		multiline_delimiter_len = q - p;
	} else {
		multiline_delimiter_len = strcspn(p, " \t");
		multiline_unescape = 1;
	}

	/* Remove trailing newline */
	multiline_delimiter_len--;
	multiline_delimiter = grecs_malloc(multiline_delimiter_len + 1);
	memcpy(multiline_delimiter, p, multiline_delimiter_len);
	multiline_delimiter[multiline_delimiter_len] = 0;
	grecs_line_begin();

	/* Update locus */
	grecs_locus_point_advance_line(grecs_current_locus_point);
	grecs_grecs_lloc.beg = grecs_current_locus_point;
	grecs_grecs_lloc.beg.col++;
}

/*
 * Auxiliary lexer functions.
 */
static int
ident(void)
{
	char *p;
	char *str;
	size_t len;
  
	for (p = grecs_grecs_text; *p && isspace(*p); p++)
		;

	len = strlen(p);
	str = grecs_malloc(len + 1);
	strcpy(str, p);
	grecs_grecs_lval.string = str;
	return IDENT;
}

static void
qstring_locus_fixup(void)
{
	if (grecs_parser_options & GRECS_OPTION_ADJUST_STRING_LOCATIONS) {
		grecs_grecs_lloc.beg.col++;
		grecs_grecs_lloc.end.col--;
	}
}

grecs_value_t *
grecs_value_ptr_from_static(grecs_value_t *input)
{
	grecs_value_t *ptr = grecs_malloc(sizeof(*ptr));
	*ptr = *input;
	return ptr;
}

/*
 * Syncline directive parsers.
 */

static int
assign_locus(struct grecs_locus_point *ploc, char *name, char *line)
{
	char *p;
	
	if (name) {
		ploc->file = grecs_install_text(name);
	}
	ploc->line = strtoul(line, &p, 10);
	ploc->col = 0;
	return *p != 0;
}

void
grecs_parse_line_directive(char *text, grecs_locus_t *ploc,
			   struct grecs_locus_point *ppoint)
{
	int rc = 1;
	struct wordsplit ws;
	
	if (wordsplit(text, &ws, WRDSF_DEFFLAGS))
		grecs_error(ploc, 0, _("cannot parse #line line: %s"),
			    wordsplit_strerror(&ws));
	else if (ws.ws_wordc == 2)
		rc = assign_locus(ppoint, NULL, ws.ws_wordv[1]);
	else if (ws.ws_wordc >= 3) 
		rc = assign_locus(ppoint, ws.ws_wordv[2], ws.ws_wordv[1]);
	else 
		grecs_error(ploc, 0, _("invalid #line statement"));
	
	if (rc) 
		grecs_error(ploc, 0, _("malformed #line statement"));
	wordsplit_free(&ws);
}

void
grecs_parse_line_directive_cpp(char *text, grecs_locus_t *ploc,
			       struct grecs_locus_point *ppoint)
{
	struct wordsplit ws;
	
	if (wordsplit(text, &ws, WRDSF_DEFFLAGS))
		grecs_error(ploc, 0, _("cannot parse #line line: %s"),
			    wordsplit_strerror(&ws));
	else if (ws.ws_wordc < 3)
		grecs_error(ploc, 0, _("invalid #line statement"));
	else if (assign_locus(ppoint, ws.ws_wordv[2], ws.ws_wordv[1]))
		grecs_error(ploc, 0, _("malformed #line statement"));
	wordsplit_free(&ws);
}

/*
 * External preprocessor support.
 */
const char *grecs_preprocessor = NULL; /* Preprocessor command */
int grecs_log_to_stderr = 1;           /* Log to stderr or to syslog? */  
void (*grecs_log_setup_hook) () = NULL;/* Log subsystem setup routine. */

static int
preproc_file(const char *file)
{
	size_t size = 0;
	char *cmd = NULL;
	char *setup_file;
	char *argv[3];
	int rc;
	
	setup_file = grecs_find_include_file("pp-setup", 0);
	if (setup_file)
		rc = grecs_asprintf(&cmd, &size,
				    "%s \"%s\" \"%s\"",
				    grecs_preprocessor, setup_file, file);
	else
		rc = grecs_asprintf(&cmd, &size,
				    "%s \"%s\"", grecs_preprocessor, file);
	if (rc)
		grecs_alloc_die();
	if ((argv[0] = getenv("SHELL")) == NULL)
		argv[0] = "/bin/sh";
	argv[1] = "-c";
	argv[2] = cmd;
	argv[3] = NULL;
	execvp(argv[0], argv);
	return -1;
}

FILE *
grecs_preproc_extrn_start(const char *file_name, pid_t *ppid)
{
	int pout[2];
	pid_t pid;
	int i;
	FILE *fp = NULL;
	int rc;

	if (pipe(pout)) {
		grecs_error(NULL, errno, "pipe");
		return NULL;
	}
	switch (pid = fork()) {
		/* The child branch.  */
	case 0:
		if (pout[1] != 1) {
			if (dup2(pout[1], 1) == -1) {
				grecs_error(NULL, errno, "dup2");
				exit(127);
			}
		}

		if (!grecs_log_to_stderr) {
			int p[2];
			char *buf = NULL;
			size_t size = 0;
			FILE *fp;
			
			signal(SIGCHLD, SIG_DFL);
			if (pipe(p)) {
				grecs_error(NULL, errno, "pipe");
				exit(127);
			}
			switch (pid = fork()) {
				/* Grandchild */
			case 0:
				if (p[1] != 2 && dup2(p[1], 2) == -1) {
					grecs_error(NULL, errno, "dup2");
					exit(127);
				}
				close(p[0]);

				rc = preproc_file(file_name);
				break;
				
			case -1:
				/*  Fork failed */
				if (grecs_log_setup_hook)
					grecs_log_setup_hook();
				grecs_error(NULL, errno, _("Cannot run `%s'"),
					    grecs_preprocessor);
				exit(127);
				
			default:
				/* Sub-master */
				close (p[1]);
				fp = fdopen(p[0], "r");
				if (grecs_log_setup_hook)
					grecs_log_setup_hook();
				while (grecs_getline(&buf, &size, fp) > 0) {
					size_t n = strlen(buf);
					if (buf[n-1] == '\n')
						buf[n-1] = 0;
					grecs_error(NULL, 0, "%s", buf);
				}
				wait(&i);
				if (WIFEXITED(i))
					rc = WEXITSTATUS(i);
				else
					rc = 127;
			}
		} else {
			rc = preproc_file(file_name);
		}
		if (rc == -1)
			exit(127);
		exit(rc);

	case -1:
		/*  Fork failed */
		grecs_error(NULL, errno, _("Cannot run `%s'"),
			    grecs_preprocessor);
		break;
		
	default:
		close(pout[1]);
		fp = fdopen(pout[0], "r");
		break;
	}
	*ppid = pid;
	return fp;
}

int
grecs_preproc_extrn_shutdown(pid_t pid)
{
	int status;
	waitpid(pid, &status, 0);
	if (WIFEXITED(status)) {
		if (WEXITSTATUS(status)) {
			grecs_error(NULL, 0, _("%s exited with status %d"),
				    grecs_preprocessor, WEXITSTATUS(status));
			return -1;
		}
	} else if (WIFSIGNALED(status)) {
		grecs_error(NULL, 0, _("%s terminated on signal %d"),
			    grecs_preprocessor, WTERMSIG(status));
		return -1;
	} else {
		grecs_error(NULL, 0, _("%s terminated abnormally (%d)"),
			    grecs_preprocessor, status);
		return -1;
	}
	return 0;
}

int
grecs_preprocess(char const *name, int trace)
{
	int rc;
	
	grecs_grecs__flex_debug = trace;
	grecs_grecs_out = stdout;
	grecs_line_acc_create();
	initial_state = PP;
	rc = push_source(name, 1);
	if (rc == 0) {
		BEGIN(PP);
		rc = grecs_grecs_lex();
		grecs_lex_end(rc);
	}
	return rc;
}

/*
 * Additional function.
 *
 * Not used by the lexer, but retained for backward compatibility.
 */
ssize_t
grecs_getline(char **pbuf, size_t *psize, FILE *fp)
{
	char *buf = *pbuf;
	size_t size = *psize;
	ssize_t off = 0;
  
	if (!buf) {
		size = 1;
		buf = grecs_malloc(size);
	}
	
	do {
		if (off == size - 1) {
			size_t nsize = 2 * size;
			if (nsize < size)
				grecs_alloc_die();
			buf = grecs_realloc(buf, nsize);
			size = nsize;
		}
		if (!fgets(buf + off, size - off, fp)) {
			if (off == 0)
				off = -1;
			break;
		}
		off += strlen(buf + off);
	} while (buf[off - 1] != '\n');

	*pbuf = buf;
	*psize = size;
	return off;
}
					

	

